import java.io.*;
import java.util.*;
import java.util.regex.*;
import cj.math.jama.*;

//  The primary use for this
//  class will be to control the sets
//  of data and gradients as well as 
//  the exclusion/inclusion of data
//  based on the removal of gradient
//  directions.
public class DataGrad
{
	//  Data list
	private ArrayList<PARREC> parrec = new ArrayList<PARREC>(1);

	//  Gradient list
	private ArrayList<Gradients> gradients = new ArrayList<Gradients>(1);

	//  TF gradients to ignore
	//  the inner most index is slice
	//  the outer most index is gradient direction
	private ArrayList<GradientsIgnore> gradientsIgnore = new ArrayList<GradientsIgnore>(1);

	final public void add(PARREC p, Gradients g)
	{
		parrec.add( p );
		gradients.add( g );
		gradientsIgnore.add( new GradientsIgnore( p.getNSlices(), g.getGradients().getRowDimension() ) );
	}

	final public void add(PARREC p, Gradients g, String numberList)
	{
		parrec.add( p );
		gradients.add( g );
		gradientsIgnore.add( new GradientsIgnore( p.getNSlices(), g.getGradients().getRowDimension(), numberList) );
	}

	final public int getNRows()
		{ return parrec.get(0).getNRows(); }

	final public int getNCols()
		{ return parrec.get(0).getNCols(); }

	final public int getNSlices()
		{ return parrec.get(0).getNSlices(); }

	final public double[] getBValues()
	{
       int tttt = 0;
        for(int ii=0; ii<parrec.size(); ii++) { tttt += parrec.get(ii).getBValues().length; }
        double[] btemp = new double[ tttt ];
        int bindex = 0;
        for(int ii=0; ii<parrec.size(); ii++)
        { 
            System.arraycopy( parrec.get(ii).getBValues(), 0, btemp, bindex,  parrec.get(ii).getBValues().length);   
            bindex +=  parrec.get(ii).getBValues().length;
        }

		return btemp;
	}

	final public double getPercentileThreshold( double pt )
	{
		return parrec.get(0).getPercentileThreshold(pt);
	} 

	final public double getReference(int row, int col, int slice)
	{
		//
		// First Compaute the Reference data.
		//
		double ref = 0.0; int num = 0;
		for(int pi=0; pi<parrec.size(); pi++)
		{
			double[] data = parrec.get(pi).getData(row,col,slice);

			//  Now get the reference indices
			int[] refinds = gradients.get(pi).getReferenceIndices();

			for(int ii=0; ii<refinds.length; ii++) 
			{
				if( ! gradientsIgnore.get(pi).ignoreQ(slice, refinds[ii] ) )
				{
					ref += data[ refinds[ ii ] ];
					num++;
				}
			}
		}
		ref /= (double)num;

		return ref;
	}

	final public double[] getData(int row, int col, int slice)
	{
		//
		// First Compaute the Reference data.
		//
		double ref = getReference(row,col,slice);

		//
		//  Now we need to create the list of data
		//
		ArrayList<Double> dd = new ArrayList<Double>(parrec.size()); //BL 10/4/06
		for(int pi=0; pi<parrec.size(); pi++)
		{
			double[] data = parrec.get(pi).getData(row,col,slice);

			//  Now get the reference indices
			int[] dirinds = gradients.get(pi).getDirectionIndices();

			for(int ii=0; ii<dirinds.length; ii++) 
			{
				if( ! gradientsIgnore.get(pi).ignoreQ(slice, dirinds[ii] ) )
				{
					dd.add( data[ dirinds[ ii ] ] / ref );
				}
			}
		}

		// Now create the output vector depending on the size of dd
		double[] output = new double[ dd.size() ];
		for(int ii=0; ii<output.length; ii++)
			output[ii] = dd.get(ii);

		return output;
	}

	//  The reason this takes a slice argument is that there
	//  is an option to remove certain gradients on a slice-by-slice
	//  bassis.
	final public Matrix getGradients(int slice)
	{
		// First we need to find out how many
		// non-ignore directions there are 
		int nn = 0;
		for(int pi=0; pi<parrec.size(); pi++)
		{
			//  Now get the reference indices
			int[] dirinds = gradients.get(pi).getDirectionIndices();

			for(int ii=0; ii<dirinds.length; ii++) 
			{
				if( ! gradientsIgnore.get(pi).ignoreQ(slice, dirinds[ii] ) )
				{
					nn++;
				}
			}
		}

		//  Now we need to fill in the output matrix
		Matrix output = new Matrix(nn, 3);
		int row = 0;
		for(int pi=0; pi<parrec.size(); pi++)
		{
			Matrix temp = gradients.get(pi).getGradients();

			//  Now get the reference indices
			int[] dirinds = gradients.get(pi).getDirectionIndices();

			for(int ii=0; ii<dirinds.length; ii++) 
			{
				if( ! gradientsIgnore.get(pi).ignoreQ(slice, dirinds[ii] ) )
				{
					output.set(row, 0, temp.get( dirinds[ii], 0));
					output.set(row, 1, temp.get( dirinds[ii], 1));
					output.set(row, 2, temp.get( dirinds[ii], 2));
					row++;
				}
			}
		}

		return output;
	}

	final public Matrix getGradients(int row, int col, int slice)
		{ return getGradients(slice); }

}

class GradientsIgnore
{
	private boolean[][] gradientsIgnore = null;

	public GradientsIgnore(int nslices, int ndirections)
	{
		init(nslices, ndirections);
	}

	public GradientsIgnore(int nslices, int ndirections, String list)
	{
		init(nslices, ndirections);

		ignoreGradients(list);
	}

	private void init( int nslices, int ndirections )
	{
		//  Now setup the gradient list to ignore
		//  and default it to ignore nothing,
		gradientsIgnore = new boolean[nslices][];
		for( int ii=0; ii<nslices; ii++)
		{
			gradientsIgnore[ii] = new boolean[ndirections];
			Arrays.fill( gradientsIgnore[ii], false);
		}
	}

	final public boolean ignoreQ(int slice, int dir )
	{
		return gradientsIgnore[slice][dir];
	}

	final public void ignoreGradients(String list)
	{
        if(list==null)
                return;
        
        //  This will be a list of
		int[][] toignore = parseNumberList( list );

		for(int sl=0; sl<toignore.length; sl++)
		{
			for(int gi=0; gi<toignore[sl].length; gi++)
			{
				gradientsIgnore[ sl	][ toignore[sl][gi] ] = true; 
			}
		}
	}

	/*
	 *  What I want to do is create a two dimensional
	 *  array that is called toRemove[slices][gradients]
	 *
	 *  For each slice, there are a list of gradients
	 *  that will be removed from the calculation.
	 *  If toRemove[slice] is null then we don't remove
	 *  any gradients.
	 */
	private int[][] parseNumberList(String nums)
	{
		int nslices = gradientsIgnore.length;

		//  Setup the output arrays
		ArrayList< ArrayList< Integer > > arrlist = new ArrayList< ArrayList< Integer > >(nslices);
		int[][] toRemove = new int[nslices][];
		
		for(int sl=0; sl<nslices; sl++)
		{
			arrlist.add(sl, new ArrayList<Integer>(1));	
		}
		
		Pattern comma = Pattern.compile(",");
		Pattern colon = Pattern.compile(":");

		/*
		 *  First thing to do is to split it on the commas.
		 *
		 *  So, each thing will be either:
		 *     1)  <numbers> 
		 *     2)  <numbers>:<numbers>
		 *     3)           :<numbers>
		 *
		 *  where <numbers> is either a number or <number>-<number>
		 *
		 *  The interpretation is:
		 *      the thing before the colon are the gradients
		 *      the thing after the colon are the slices
		 */
		String[] things = comma.split( nums );

		for(int ii=0; ii<things.length; ii++)
		{
			things[ii] = things[ii].trim();
			
			String[] colond = colon.split(things[ii]);
			
			//  If there is only one thing here then
			//  the person has said the want to remove
			//  the whole gradient (i.e., for every slice)
			if( colond.length == 1 )
			{
				int[] gradients = parseDash( colond[0] );
				
				for(int sl=0; sl<nslices; sl++)
				{
					for(int gi=0; gi<gradients.length; gi++)
					{
						arrlist.get(sl).add(gradients[gi]);
					}
				}
			}
			//  If there are two things then we want to 
			//  remove 
			else if ( colond.length == 2 )
			{
				int[] gradient = parseDash( colond[0] );
				int[] slices = parseDash( colond[1] );
			
				for(int sl=0; sl < slices.length; sl++)
				{
					for (int gi=0; gi < gradient.length; gi++)
					{
						arrlist.get(slices[sl]).add( gradient[gi]);	
					}
				}
			}
			else 
			{
				System.out.println(nums + " is not properly formatted.");
				System.exit(0);
			}
			
		}
		
		// And finally...
		for(int sl=0; sl < nslices; sl++)
		{
//			System.out.println("Removeing the gradients for slice " + sl);
			int[] grads = new int[ arrlist.get(sl).size() ];
			for(int ii=0; ii< arrlist.get(sl).size(); ii++)
			{
				grads[ii] = arrlist.get(sl).get(ii);
//				System.out.print(grads[ii] + ", ");
			}
//			System.out.println("");
			
			toRemove[sl] = new int[ grads.length ];
			System.arraycopy( grads, 0, toRemove[sl], 0, grads.length);
		}
		
		return toRemove;
	}
	
	/*
	 *  The argument to this function must be 
	 */
	final public int[] parseDash(String arg)
	{
		int[] toreturn = null;

		//  Going to split on the dash
		Pattern dash = Pattern.compile("-");		
		String[] things = dash.split( arg );
		
		// If there is one thing, i.e., no dash
		if( things.length == 1 )
		{
			toreturn = new int[1];
			
			try 
			{
				toreturn[0] = (new Integer( things[0] )).intValue();
			}
			catch( NumberFormatException e )
			{
				System.out.println("Nope, " + arg + " is not a number.");
				System.exit(0);
			}
		}  // If there are two things, i.e.,  10-22
		else if( things.length == 2 )
		{
			int start=0, end=0;
			try
			{
				start = (new Integer( things[0] )).intValue();			
			}
			catch( NumberFormatException e )
			{
				System.out.println("Nope, " + arg + " is not a number.");
				System.exit(0);
			}

			try 
			{
				end = (new Integer( things[1] )).intValue();
			}
			catch( NumberFormatException e )
			{
				System.out.println("Nope, " + arg + " is not a number.");
				System.exit(0);
			}
			
			toreturn = new int[ end-start+1 ];
			
			for(int ii=start,jj=0; ii<=end; ii++)
			{
				toreturn[jj] = ii;
				jj++;
			}
		}
		else 
		{
			System.out.println("Could not parse " + arg);
			System.exit(0);
		}
		
		return toreturn;
	}

}
