package apps.autogrow;

import cj.appkit.*;
import cj.volume.*;
import cj.volume.measure.*;
import cj.roi.*;
import cj.math.convexp.*;
import cj.math.matrix.*;
import cj.fileformats.*;
import java.util.Arrays.*;

final public class AutoGrowApp {
    public static void main(String argv[]) {
        AppKit myApp = new AutoGrowKit(); myApp.main(argv);
    }
}

final class AutoGrowKit extends AppKit {
	private boolean compatMode = false;

    public String getAppName() { return "autogrow"; }
    public String getDialogName() { return "Load dual MIF"; }

    public void help() {
        System.out.println("autogrow [options] C0910D01.MIF ...");
        printOptions();
        System.out.println("Basis points are loaded from C0910S01.roi.");
 		System.out.println("Grown lesions are saved to C0910S01G.roi.");
    }

protected void moreOpts()
{
	addOpt_noArg("les",'l',"Use lesion points as grow points (old style).");
}

protected boolean parseArg(int c)
{
	switch(c) {
		case 'l':
			compatMode = true;
			break;
		default:
			return false;
	}
	return true;
}

final public void run()
{
if (verbose > 0)
{
	if (compatMode) System.out.println("Using compatibility mode.");
	else System.out.println("Using lesion grow point mode.");
}

String argv[] = otherArgs;

for(int ii=0;ii<argv.length;ii++) {
	String MIFname = argv[ii]; int l = MIFname.length();
	String pre = MIFname.substring(0,l-7);
	String post = MIFname.substring(l-6,l-4);
	String ROIin = pre + "S" + post + ".roi";
	String ROIout = pre + "S" + post + "G.roi";

	if (verbose > 0)
		System.out.println("Reading "+MIFname+"...");

	MRIFile mf = MRIFile.readMRIFile(MIFname);
	ROIList rl = new ROIList(ROIin);
	MCVolume v = mf.getMCVolume();
	ConvexP cp = new ConvexP(4,2);
	String[] tl = {"les","csf","gm","wm"};

	RegionGrow rg = new RegionGrow(v,cp,rl,tl);

	ROI[] R = rl.findAllUser(compatMode?"les":"les.g");
	POI p = new POI();
	
	if (verbose > 0)
		{
		System.out.print("Found "+R.length+" lesion ");
		System.out.println(compatMode?"points.":"grow points.");
		}

	for(int i=0; i<R.length; i++)
	{
		R[i].getPOI(0,p);
		ROI r = rg.grow(p.x, p.y, p.z, "les");

        if (r == null) break;

		int n = r.size();

		if (verbose > 0)
		{
			if (n<5) System.out.print(".");
			else if (n<20) System.out.print(",");
			else if (n<100) System.out.print(":");
			else if (n<500) System.out.print(";");
			else if (n<2000) System.out.print("=");
			else System.out.print("#");
			System.out.flush();
		}

		if (n < 5) continue;

		if (!compatMode)
		{
			boolean already = false;
			String s1 = r.getComments();

			ROI[] RP = rl.findAllWithComment("les",s1);
			for(int j=0; j<RP.length; j++)
			{
			// THIS NEEDS TO CHANGE WHEN ROI TRANSPOSE BUG IS FIXED
				if (RP[j].contains(p.y,p.x,p.z))
					{already = true; break;}
			}

			if (already) continue; // Don't add if already there.

			r.setAssociation(R[i]);
		}

		rl.add(r);
	}

	if (verbose > 0) System.out.println("\nDone!");
	if (verbose > 1) AppKit.memStatus(System.out,true);

	rl.write(ROIout);

	}

}

}
