package apps.diffusion;

import cj.volume.*;
import cj.mri.diffusion.*;
import cj.appkit.FilterAppKit;

final public class DiffusionApp {
   public static void main(String argv[]) { new DiffusionKit().main(argv); }
}

final class DiffusionKit extends FilterAppKit {

	private boolean famap = false;
	private String gradient_file = "grad.txt";
	private String output_suffix = "S";
    private String comment = "";

	public String getAppName() { return "diffusion"; }
	public String getDialogName() { return "Process image"; }
    public String getOutputSuffix() { return output_suffix; }
    public String getComment() { return comment; }

	private Diffusion diff = new Diffusion();
	private DiffusionStatistics diff_stats;

    protected void moreOpts() { 
        addOpt_noArg("fa", 'f', "Calculate FA map");
        addOpt_reqArg("gradient-file", 'g', "Gradient file (def: grad.txt).");
        addOpt_reqArg("output-suffix", 800, "Output suffix (def: 'S').");
    }

    protected boolean parseArg(int c)
    {
    	switch(c) {
			case 'f':
				famap = true;
				break;
			case 'g':
				gradient_file = getStringArg("gradient file");
				break;
			case 800:
				output_suffix = getStringArg("suffix");
				break;
			default:
				return false;
		}
		return true;

    }

	final public void initParams(MCVolume v)
	{
		diff.readGradientFile(gradient_file);
		diff_stats = new DiffusionStatistics(diff.getD());
	}

    final public void printArgs()
   	{
		System.out.println("Gradient file: " + gradient_file);
   	}

    final public MCVolume filter(MCVolume v)
	{
//		diff.calculateFA(v);

		long begin = System.currentTimeMillis();
		System.out.println("Starting...");
		for(int row=0; row < v.getNRows(); row++) {
		for(int col=0; col < v.getNCols(); col++) {
		for(int slice=0; slice < v.getNSlices(); slice++) {
		if( v.getData(row,col,slice,0) > 60 ) {
		double[] tt = v.getChannelDouble(row, col, slice);
		diff.computeTensor( tt );
		}}}}
		System.out.println("Done.");
		long end = System.currentTimeMillis();
		System.out.println("Took " + (end-begin) + " ms" );

		return v;
	}
}
