package apps.displaysl;

import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import cj.fileformats.*;
import cj.volume.*;
import cj.appkit.*;
import cj.utilities.MIFName;

final public class DisplaySLApp {
	public static void main(String argv[]) { new DisplaySLKit().main(argv); }
}

final class DisplaySLKit extends AppKit {

	Vector lays = new Vector();

	public String getAppName() { return "mrdual"; }
	public String getDialogName() { return "Show image"; }

	public void moreOpts()
	{
		addOpt_reqArg("roi",'r',"display another volume as overlay");
	}

	public boolean parseArg(int c)
	{
		switch(c) {
		case 'r':
			lays.add(getStringArg("overlay"));
			break;
		default:
			return false;
		}
		return true;
	}

	public void help() {
		System.out.println("mrdual [options] [{filename|directory}]");
		printOptions();
		System.out.println("This loads a short/long pair in MIF3/4 format to explore slice drift.");
	}

	public void run() {
		for (int i = 0 ; i < otherArgs.length ; i++) {
			if (verbose > 0) {System.out.println("Loading " + otherArgs[i]);}
			doit(i, otherArgs[i]);
		}
	}

public void doit(int index, String filename) {
    String filename_s=null, filename_l=null;

	{
		MIFName mn = new MIFName(filename);
		if (mn.isValid())
		{
			filename_s = mn.recode('S').toString();
			filename_l = mn.recode('L').toString();
		}
		else
		{
			System.err.println("Couldn't validate "+filename);
			System.exit(-1);
		}
	}

	{
		//-------------------------------------------------------
		//
		//  Bring up a dialog to say that we are loading.
		//
		JFrame loading_frame = new JFrame();
		JTextArea loading_dialog = new JTextArea("Loading " + filename_s);
		loading_dialog.setFont(new Font("helvetica", Font.BOLD, 14));
		loading_frame.getContentPane().add( loading_dialog ); 
		loading_frame.setResizable(false);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		loading_frame.setSize(300,50);
		//loading_frame.setLocation(screenSize.width/2-150 + index*30,screenSize.height/2-25 + index*30);
		loading_frame.setLocation(screenSize.width/2-150,screenSize.height/2-25);
		loading_frame.pack();
		loading_frame.show();

		//-------------------------------------------------------
		//
		//  Load up the file.
		//
		MRIFile b = MRIFile.readMRIFile(filename_s);

		int sdate = 0, ldate = 0;

		if (b.getFileType() == MRIFile.FILETYPE_MIF4)
		{
			sdate = ((MIF4)b).GetScanDateTime();
		}

		MCVolume v = b.getMCVolume();

		loading_dialog.setText("Loading " + filename_l);

		b = MRIFile.readMRIFile(filename_l);

		if (b.getFileType() == MRIFile.FILETYPE_MIF4)
		{
			ldate = ((MIF4)b).GetScanDateTime();
		}

		v.concatenate(b.getMCVolume());

		b = null;  // Release a little memory

		if (sdate != ldate)
		{
			System.err.println("Scan dates do not match!");
			System.err.println("Short: " + sdate);
			System.err.println("Long:  " + ldate);
		}

		loading_dialog.setText("Initialising...");
		MCVolumePanel dm = new MCShiftPanel(v);
		int nRows=v.getNRows(), nCols=v.getNCols();

		//-------------------------------------------------------
		//
		//  Destroy the dialog to say that we are loading.
		//
		loading_frame.dispose();
		loading_frame = new JFrame(filename_s);
		
		loading_frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

		dm.setSize(nRows, nCols);
		loading_frame.getContentPane().add(dm);
		loading_frame.addKeyListener(dm);
		loading_frame.getContentPane().addMouseListener(dm);
		loading_frame.getContentPane().addMouseMotionListener(dm);
		loading_frame.setResizable(true);
		loading_frame.setSize(2*nCols+2,2*nRows+40);
		loading_frame.setLocation(index*256,0);

        for(int i=0;i<lays.size();i++)
        {
            String s=(String)lays.get(i);
            Volume ov = MRIFile.readMRIFile(s).getMCVolume().getVolume(0);
            char ch; Color c;

            switch(i) {
                case 0:
                    ch = 'b'; c = new Color(255,0,0,128);
                    break;
                case 1:
                    ch = 'z'; c = new Color(0,255,0,128);
                    break;
                case 2:
                    ch = 'B'; c = new Color(0,0,255,128);
                    break;
                default:
                    ch = 'Z'; c = new Color(255,255,255,128);
                    break;
            }
            dm.addOverlay(ov, c, ch);
        }

		loading_frame.show();
	}
}
}
