package apps.mrdiff;

import cj.appkit.*;
import cj.volume.*;
import cj.volume.measure.*;
import cj.utilities.Format;

final public class MRDiffApp {
	public static void main(String argv[]) { new MRDiffKit().main(argv); }
}

class MRDiffKit extends FilterAppKit {

	boolean calc_mi = false;
	boolean calc_diff = false;
	boolean calc_vr = false;

	boolean first_run=true;
	String firstFile;
	MCVolume v1=null, v2=null;

	public void pre() { filterMode = READ_ONLY; }

	public String getAppName() { return "mrdiff"; }
	public String getDialogName() { return "Compare image"; }

    protected void moreOpts()
	{
		addOpt_noArg("mutual",'u',"Calculate mutual information");
		addOpt_noArg("diff",'d',"Calculate mean absolute difference");
		addOpt_noArg("woods",'w',"Calculate variance of ratios");
		addOpt_noArg("all",'a',"Calculate all of the above");
	}

	public void initParams(MCVolume v) {}
	public void printArgs() {}

	protected boolean parseArg(int c)
	{
		switch(c) {
			case 'u':
				calc_mi = true;
				break;
			case 'd':
				calc_diff = true;
				break;
			case 'w':
				calc_vr = true;
				break;
			case 'a':
				calc_mi = calc_diff = calc_vr = true;
				break;
			default:
				return false;
		}
		return true;
	}

	public MCVolume filter(MCVolume dm) {
		//
		//  Set kappa based on the noise estimation if it 
		//  was not specified as a parameter.
		//
		if ( first_run )
		{
			firstFile = new String(filename); v1 = dm;
			first_run = false;
			return dm;
		}

		v2 = dm;

		Format.print(System.out, "%15s: " +firstFile+ " to " +filename+ "\n", "Comparing");

		if( calc_mi )
		{
			double[] lb={0,0}, ub = new double[2], bins = new double[2];
			ub[0] = Statistics.max(v1); ub[1] = Statistics.max(v2);
			bins[0] = ub[0]/10; bins[1] = ub[1]/10;

			Hist2D h = new Hist2D(lb,ub,bins); h.clear();
			for(int ch=0;ch<v1.getNChannels();ch++)
				h.add(v1.getVolume(ch),v2.getVolume(ch));

			Format.print(System.out, "%15s: " + h.mutualInfo() + "\n", "Mutual Info");
		}

		if( calc_diff )
		{
			double t = 0;
			float[] row1, row2;

			for(int ch=0;ch<v1.getNChannels();ch++) for(int s=0;s<v1.getNSlices();s++)
				for(int r=0;r<v1.getNRows();r++)
				{
					row1 = v1.getRow(r,s,ch); row2 = v2.getRow(r,s,ch);
					for(int i=0;i<row1.length;i++)
						t += Math.abs(row1[i]-row2[i]);
				}
			t = t/v1.getNChannels()/v1.getNSlices()/v1.getNRows()/v1.getNCols();

			Format.print(System.out, "%15s: " + t + "\n", "Absolute Diff");
		}

		if( calc_vr )
		{
			final int MAXVAL=4096;
			double t = 0;
			double d1, d2;
			double[] vcf_tot = new double[MAXVAL],
                     vcf_sqr_tot = new double[MAXVAL];
            long[] vcf_count = new long[MAXVAL];

			for(int ch=0;ch<v1.getNChannels();ch++) for(int s=0;s<v1.getNSlices();s++)
				for(int r=0;r<v1.getNRows();r++) for(int c=0;c<v1.getNCols();c++)
				{
					d1 = v1.getData(r,c,s,ch); d2 = v2.getData(r,c,s,ch);
					int i = (int)d1;
					if (i >=0  && i < MAXVAL)
					{
						vcf_tot[i] += d2; vcf_sqr_tot[i] += d2*d2;
						vcf_count[i]++;
					}
				}
			
			double vcf_total_dev=0, vcf_total_count=0;

			for(int i=0;i<MAXVAL;i++)
			{
				if (vcf_count[i]==0) continue;
				double vcf_mean = vcf_tot[i]/vcf_count[i];
				double vcf_std_dev = vcf_sqr_tot[i] - vcf_mean*vcf_mean*vcf_count[i];
				//System.out.println("("+i+","+vcf_mean+","+vcf_std_dev+")");
				vcf_total_dev += vcf_std_dev;
				vcf_total_count += vcf_count[i];
			}

			t = Math.sqrt(vcf_total_dev/vcf_total_count);

			Format.print(System.out, "%15s: " + t + "\n", "Var. of Ratio");
		}

		return dm;
	}
}
