package apps.mrdisp;

import javax.swing.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import cj.fileformats.*;
import cj.volume.*;
import cj.appkit.*;
import cj.roi.*;

final public class MRDispApp {
	public static void main(String argv[]) { new MRDispKit().main(argv); }
}

final class MRDispKit extends AppKit {

    boolean doScale = false;
	Vector lays = new Vector();
	MCVolume V = null;
	JFrame loading_frame;
	Dimension screen;

	public String getAppName() { return "mrdisp"; }
	public String getDialogName() { return "Show image"; }

    public void moreOpts()
	{
		addOpt_noArg("scale",'s',"scale to fit exact range of data");
		addOpt_reqArg("overlay",'o',"display another volume as overlay");
	}

    public boolean parseArg(int c)
	{
		switch(c) {
		case 's':
			doScale = true;
			break;
		case 'o':
			lays.add(getStringArg("overlay"));
			break;
		default:
			return false;
		}
		return true;
	}

	public void help() {
		System.out.println("mrdisp [options] [{filename|directory}]");
		printOptions();
		System.out.println("This will display any MIF3/MIF4/BFF/TBF or raw data file.");
	}

	public void run() {
		loading_frame = new JFrame(otherArgs[0]);

		loading_frame.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
	
		// Set the initial location in the middle of the screen.
 		screen = Toolkit.getDefaultToolkit().getScreenSize();
		loading_frame.setSize(256+5,256+40);
		loading_frame.setLocation(screen.width/2-256/2,screen.height/2-256/2);
		

		for (int i = 0 ; i < otherArgs.length ; i++) {
			if (verbose > 0) {System.out.println("Loading " + otherArgs[i]);}
			doit(i, otherArgs[i]);
		}

		MCVolumePanel dm = new MCShiftPanel(V, false, doScale);
		loading_frame.getContentPane().add(dm);
		dm.addKeyListener(dm);
		dm.addMouseListener(dm);
		dm.addMouseMotionListener(dm);
		dm.setSize(V.getNRows(), V.getNCols());
	
		Volume ov = null;
		for(int i=0;i<lays.size();i++)
		{
			String s=(String)lays.get(i);

			if( s.toLowerCase().endsWith(".roi") || s.toLowerCase().endsWith(".les") )
			{
				VoxelInfo vi = new VoxelInfo();
				vi.xDimension = V.getNRows();
				vi.yDimension = V.getNCols();
				vi.zDimension = V.getNSlices();
				vi.xSize = 1.0; // This program doesn't need to really know
				vi.ySize = 1.0; // at this point!!!
				vi.zSize = 1.0;
				ov = RegionList.readRegionList(s, vi).getVolume();
			}
			else
			{
				ov = (MRIFile.readMRIFile(s)).getVolume(0);
			}
			char ch; Color c;
	
			switch(i) {
				case 0: 
					ch = 'R'; c = new Color(255,0,0,128);
					break;
				case 1:
					ch = 'G'; c = new Color(0,255,0,128);
					break;
				case 2:
					ch = 'B'; c = new Color(0,0,255,128);
					break;
				default:
					ch = 'Z'; c = new Color(255,255,255,128);
					break;
			}

			dm.addOverlay(ov, c, ch, s);
		}
	
		loading_frame.show();
		dm.requestFocus();
	}

	public void doit(int index, String filename) {
		JTextField waiting = new JTextField("Loading " + filename + "...");
		loading_frame.getContentPane().add(waiting);
	
		loading_frame.setResizable(true);
		loading_frame.show();
	
		//-------------------------------------------------------
		//
		//  Load up the file.
		//
		MRIFile b = MRIFile.readMRIFile(filename);
		MCVolume v = b.getMCVolume();
	
		if (index == 0)
		{
			int nRows=v.getNRows(), nCols=v.getNCols();
			loading_frame.setSize(nCols+5,nRows+40);
			loading_frame.setLocation(screen.width/2-nCols/2,screen.height/2-nRows/2);
		}
	
		b = null;  // Release a little memory
	
		if (V == null) V = v; else V.concatenate(v);

		loading_frame.getContentPane().remove(waiting);
}

}
