package apps.mrhalf;

import cj.appkit.*;
import cj.fileformats.*;
import cj.volume.*;

final public class MRHalfApp {
	public static void main(String argv[]) { new MRHalfKit().main(argv); }
}

class MRHalfKit extends FilterAppKit {

	public void pre() { filterMode = READ_WRITE; }

	public String getOutputSuffix() { return "D"; }
	public String getAppName() { return "mrhalf"; }
	public String getDialogName() { return "Downsample image"; }

	public void preSave(MRIFile b)
	{
		if (b instanceof MIF4)
		{
			// Need to figure out if other stuff needs to be changed
			MIF4 mb = (MIF4)b;
			mb.SetPixelX(mb.GetPixelX()*2);
			mb.SetPixelY(mb.GetPixelY()*2);
		}
		else
		{
			System.out.println("Warning: output header is likely incorrect!");
		}
	}

	public MCVolume filter(MCVolume dm)
	{
		int h1=dm.getNRows()/2, h2=dm.getNCols()/2,
			h3=dm.getNSlices(), h4=dm.getNChannels();

		MCVolume out = new MCVolume(h1, h2, h3, h4);

		for(int ch=0;ch<h4;ch++)
			for(int sl=0;sl<h3;sl++)
			{
				Slice si = dm.getSlice(sl,ch);
				Slice so = out.getSlice(sl,ch);

				for(int c=0;c<h2;c++)
					for(int r=0;r<h1;r++)
						{
						float t = si.getData(2*r,2*c);
						t += si.getData(2*r,2*c+1);
						t += si.getData(2*r+1,2*c);
						t += si.getData(2*r+1,2*c+1);
						t *= 0.25;
						so.setData(Math.round(t),r,c);
						}

				out.setSlice(so,sl,ch);
			}
		return out;
	}
}
