package apps.nnlsdisp;

import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;

import cj.mri.relax.t2fit.*;
import cj.mri.relax.t2bin.*;
import cj.volume.MCVolume;
import graph.*;

class DisplayMCVolume extends JFrame implements KeyListener, MouseListener, MouseMotionListener, ComponentListener, ActionListener {

	private JPanel wholePanel;
	private JPanel plotPanel;  // left side
	private JPanel dataPanel;  // right side

	private ImagePanel imagePanel;  // goes into dataPanel
	private JSlider sliceSlider;  // goes into dataPanel

	private Graph2D topChartPanel;
	private ChartPanel bottomChartPanel;
	private DataSet data1;
	private DataSet data2;
	private Axis xaxis;
	private Axis yaxis;
	private int width, height, nslices, nchannels;
	private int[] displayData = null;
	private double[] plot_decay = null;
	private double[] decay = null;
	private int[] imageData = null;
	private Point mouseLoc;
	private double[] te;

	private T2FitNNLS nnls = null;

	private JRadioButton smallNNLS, regularNNLS;

	//-----------------------------------------------------
	//
	//  Constructor
	//
	DisplayMCVolume(String title, MCVolume data, double[] te_in) {

		super();

		height = data.getNRows();
		width = data.getNCols();
		nslices = data.getNSlices();
		nchannels = data.getNChannels();
		te = new double[te_in.length];
		for(int ii=0; ii<te_in.length; ii++) {
			te[ii] = te_in[ii];
		}

		//
		//  Scale and set the imageData variable.
		//
		int[] tempData = new int[width*height*nslices*nchannels];
		int hh=0;
		for(int ll=0; ll<nchannels; ll++) {
		for(int kk=0; kk<nslices; kk++) {
		for(int jj=0; jj<height; jj++) {
		for(int ii=0; ii<width; ii++) {
			tempData[hh] = (int)data.getData(ii,jj,kk,ll);
			hh++;
		}
		}
		}
		}

		initialize(title, tempData, height, width, nslices*nchannels);
	}

	//-----------------------------------------------------
	//
	//  Constructor
	//
	DisplayMCVolume(String title, int[] id, int w, int h, int slices) {
		super();

		initialize(title, id, w, h, slices);
	}

	private void initialize(String title, int[] id, int w, int h, int slices) {

		addComponentListener(this);

		//
		//  Tells the program to exit when this application is closed.
		//
		setDefaultCloseOperation(3);

		width = w;
		height = h;
		nslices = slices;

		//
		//  Copy the data over (scaled).
		//
		int max=0; 
		for(int ii=0; ii<width*height*nslices; ii++) {
			max = Math.max( id[ii], max);
		}

		//
		//  Scale and set the imageData variable.
		//
		displayData = new int[width*height*nslices];
		imageData = new int[width*height*nslices];
		for(int ii=0; ii<width*height*nslices; ii++) {
			imageData[ii] = id[ii];
			displayData[ii] = (int)((float)id[ii] / (float)max * 255.0);
		}

		//
		//  Set the true imageData variable to RGB.
		//
		int jj;
		for(int ii=0; ii<width*height*nslices; ii++) {
			jj = displayData[ii] & 0xff;
			displayData[ii] = ((255<<24) | (jj<<16) | (jj<<8) | jj);
		}

		
		//
		//  Set the mouse and key listeners.
		//
		addKeyListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);

		//  Set the frame's border layout.
		getContentPane().setLayout(new BorderLayout(1,1));

		wholePanel = new JPanel();
//		wholePanel.setLayout(new BorderLayout(1,2));
		wholePanel.setLayout(new BoxLayout(wholePanel, BoxLayout.X_AXIS));

		//  Initialize the plotPanel (left side).
		initPlotPanel();

		//  Initialize the dataPanel (right side).
		initDataPanel();

		wholePanel.setPreferredSize(
			new Dimension(plotPanel.getPreferredSize().width + 
		                  dataPanel.getPreferredSize().width + 20 , 
		                  Math.max(plotPanel.getPreferredSize().height, 
				          dataPanel.getPreferredSize().height)));

		getContentPane().add( wholePanel );

		//
		//  Set the size and display the frame.
		//
		setSize(new Dimension(wholePanel.getPreferredSize().width,
		                 wholePanel.getPreferredSize().height));
		setTitle(title);
		setResizable(true);
		show();

		nnls = new T2FitNNLS(te, nnls.getT2());
	}

	private void initDataPanel() {

		//================================================================
		//
		//  Setup right hand panel.
		//
		dataPanel = new JPanel();
		dataPanel.setLayout(new BorderLayout(2,1));
		dataPanel.setVisible(true);

//		wholePanel.add( dataPanel, BorderLayout.EAST);
		wholePanel.add( dataPanel );

		//-------------------------------------------------
		//
		//  Create the panel.
		//
		System.out.println(width);
		imagePanel = new ImagePanel(displayData, width, height, nslices);
		imagePanel.setPreferredSize(new Dimension(2*width+20, 2*height));
		imagePanel.setMinimumSize(new Dimension(2*width+20, 2*height));
		imagePanel.setBorder(BorderFactory.createTitledBorder("Image"));

		dataPanel.add(imagePanel, BorderLayout.NORTH);
		dataPanel.setBorder(BorderFactory.createTitledBorder("Data"));

		//-------------------------------------------------
		//
		//  Create the radio buttons
		//
		regularNNLS = new JRadioButton("Regular");
		regularNNLS.setMnemonic(KeyEvent.VK_R);
		regularNNLS.setActionCommand("Regular");
		regularNNLS.setSelected(true);

		smallNNLS = new JRadioButton("Small Norm");
		smallNNLS.setMnemonic(KeyEvent.VK_S);
		smallNNLS.setActionCommand("Small");

		regularNNLS.addActionListener(this);
		smallNNLS.addActionListener(this);

		ButtonGroup bg = new ButtonGroup();
		bg.add(regularNNLS);
		bg.add(smallNNLS);
		
		JPanel radioPanel = new JPanel();
		radioPanel.add(regularNNLS);
		radioPanel.add(smallNNLS);
		radioPanel.setBorder(BorderFactory.createTitledBorder("NNLS Solution"));
		radioPanel.setPreferredSize(
			new Dimension(imagePanel.getPreferredSize().width-80,80));

		dataPanel.add(radioPanel, BorderLayout.CENTER);

		//-------------------------------------------------
		//
		//  Create the slider.
		//
		sliceSlider = new JSlider(JSlider.HORIZONTAL, 1, nslices, 1);
		sliceSlider.setPreferredSize(
			new Dimension(imagePanel.getPreferredSize().width-80,80));
		sliceSlider.setBorder(BorderFactory.createTitledBorder("Slice Number"));

		sliceSlider.setMajorTickSpacing(5);
		sliceSlider.setMinorTickSpacing(1);
		sliceSlider.setPaintTicks(true);
		sliceSlider.setPaintLabels(true);
		sliceSlider.addChangeListener(new SliceListener());

		dataPanel.add(sliceSlider, BorderLayout.SOUTH);

		//
		//  Set the dataPanel's sizes.
		//
		dataPanel.setPreferredSize(new Dimension(imagePanel.getPreferredSize().width + 80,
			imagePanel.getPreferredSize().height + 
			radioPanel.getPreferredSize().height +
			sliceSlider.getPreferredSize().height));
	}

	private void initPlotPanel() {

		//================================================================
		//
		//  Setup left hand panel.
		//
		plotPanel = new JPanel();
		plotPanel.setLayout( new BoxLayout(plotPanel, BoxLayout.Y_AXIS) );
		plotPanel.setBorder(BorderFactory.createTitledBorder("Plots"));

		//-------------------------------------------------
		//
		//  Create the topChartPanel
		//
		topChartPanel = new Graph2D();
		topChartPanel.setPreferredSize(new Dimension( 200, 200 ) );
		topChartPanel.setBorder(BorderFactory.createTitledBorder("T2 Decay Curve"));
		topChartPanel.drawzero = false;
		topChartPanel.drawgrid = false;
		topChartPanel.borderTop = 50;

		decay = new double[te.length];
		plot_decay = new double[2*te.length];
		int i,j;
		for(i=j=0; i<32; i++,j+=2) {
			plot_decay[j] = te[i];
			plot_decay[j+1] = 0.0;
		}

		data1 = topChartPanel.loadDataSet(plot_decay,32);
		data1.linecolor = new Color(0,0,128);

		data2 = topChartPanel.loadDataSet(plot_decay,32);
		data2.linestyle = 0;
		data2.marker    = 1;
		data2.markerscale = 1.5;
		data2.markercolor = new Color(0,0,128);

		xaxis = topChartPanel.createAxis(Axis.BOTTOM);
		xaxis.attachDataSet(data1);
		xaxis.attachDataSet(data2);
		xaxis.setTitleText("T2 (ms)");
		xaxis.setTitleFont(new Font("TimesRoman",Font.PLAIN,14));
		xaxis.setLabelFont(new Font("Helvetica",Font.PLAIN,12));

		yaxis= topChartPanel.createAxis(Axis.LEFT);
		yaxis.attachDataSet(data1);
		yaxis.attachDataSet(data2);
		yaxis.setTitleText("Signal Intensity");
		yaxis.setTitleFont(new Font("TimesRoman",Font.PLAIN,14));
		yaxis.setLabelFont(new Font("Helvetica",Font.PLAIN,12));


		//-------------------------------------------------
		//
		//  Create the bottomChartPanel
		//
		bottomChartPanel = new ChartPanel();
		bottomChartPanel.setPreferredSize(new Dimension( 256, 256) );
		bottomChartPanel.setBorder(BorderFactory.createTitledBorder("T2"));

		bottomChartPanel.setXUpper( 200 );


		plotPanel.add(topChartPanel);
		plotPanel.add(bottomChartPanel);
	
		//
		//  Set the plotPanel's size.
		//
		plotPanel.setPreferredSize(new Dimension(
			Math.max( topChartPanel.getPreferredSize().width, 
				      bottomChartPanel.getPreferredSize().width), 
			topChartPanel.getPreferredSize().height + 
			bottomChartPanel.getPreferredSize().height + 80));

		wholePanel.add( plotPanel );
	}

	private void doNNLS( int x, int y ) {

		int i,j;

		System.out.println("(" + x + ", " + y + ")");

		imagePanel.displayMarker(x, y);

		//  Find out the data for the echo and pass it to 
		//  the plot channel.

		int xo2 = x/2;
		int yo2 = y/2;
		int wh = width*height;
		for(i=j=0; i<32; i++,j+=2) {
			plot_decay[j] = te[i];
			plot_decay[j+1] = (double)imageData[i*wh+ width*yo2 + xo2];
			decay[i] = (double)imageData[i*wh+ width*yo2 + xo2];
		}

		try {
			data1.deleteData();
			data1.append(plot_decay, 32);
			data2.deleteData();
			data2.append(plot_decay, 32);
		}
		catch( Exception exj )
		{
			System.out.println("Could not delete or append data");
		}
		plotPanel.repaint();

		//
		//  Call NNLS.
		//
		T2Distribution dist = new T2Distribution();
		nnls.solve(decay, dist);
		double[] t2s = dist.getT2s();
		double[] amplitudes = dist.getAmplitudes();
		bottomChartPanel.setXY(t2s, amplitudes);
	}

	protected void paintComponent(Graphics g) {
		imagePanel.paintComponent(g);
	}

	//-----------------------------------------------------
	//
	//  Mouse Listeners
	//
    public void mouseReleased(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {
		int x, y;

        mouseLoc = e.getPoint();

		x = mouseLoc.x - dataPanel.getX() - 2*imagePanel.getX();
		y = mouseLoc.y - dataPanel.getY() - 2*imagePanel.getY();

		System.out.println("mouse loc " + mouseLoc.x + " " + mouseLoc.y );
		System.out.println("datapanel " + dataPanel.getX() + " " + dataPanel.getY() );
		System.out.println("imagepanel" + imagePanel.getX() + " " + imagePanel.getY() );

		if( x > -1 && x < 512 && 
		    y > -1 && y < 512 ) {
			doNNLS(x, y);
		}
    }

    public void mouseExited(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}

	public void mouseDragged(MouseEvent e) {
		int x, y;

        mouseLoc = e.getPoint();

		x = mouseLoc.x - dataPanel.getX() - 2*imagePanel.getX();
		y = mouseLoc.y - dataPanel.getY() - 2*imagePanel.getY();

		if( x > -1 && x < 512 &&
		    y > -1 && y < 512 ) {
			doNNLS( x, y );
		}
	}

	public void mouseMoved(MouseEvent e) {}

	public void keyPressed(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {
        //System.out.println("The " + e.getKeyChar() + 
		//	                     " key pressed (" + mouseLoc.x + 
		//					  ", " +mouseLoc.y+").");

		if( e.getKeyChar() == 'q' || e.getKeyChar() == 'Q' ) {
			System.exit(0);
		}
	}

	class SliceListener implements ChangeListener {
		
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			imagePanel.displaySlice(source.getValue());
		}
	}

	public void componentHidden(ComponentEvent e) {
	}

	public void componentMoved(ComponentEvent e) {
	}

	public void componentResized(ComponentEvent e) {
		Dimension curr = getPreferredSize();
		dataPanel.setPreferredSize( new Dimension( curr.width/2, curr.height ) );
		plotPanel.setPreferredSize( new Dimension( curr.width/2, curr.height ) );
	}

	public void componentShown(ComponentEvent e) {
	}

	public void actionPerformed(ActionEvent e) {
		if( e.getActionCommand().compareTo("Regular") == 0 ) {
			nnls.setTE(te);
//			nnls.init(te, nnls.getT2());
		}
		else {
//			nnls.create(te, nnls.getT2(), 1.02, 1.025);
		}
	}
}

/* class ImagePanel extends JPanel implements MouseListener { */
class ImagePanel extends JPanel {

	private MemoryImageSource mi;
	private Image m_image;
	private int width, height, nslices;
	private int imageData[];
	private int marker_x, marker_y;
	private Font font;

	ImagePanel(int[] p_imageData, int p_width, int p_height, int p_nslices) {

		//
		//  Copy the width and height to internal variables.
		//
		width = p_width;
		height = p_height;
		nslices = p_nslices;

		imageData = new int[width*height*nslices];

		font = new Font("helvetica", Font.PLAIN, 12);

		//
		//  Copy the data over.
		//
		System.arraycopy(p_imageData, 0, imageData, 0, width*height*nslices);

		mi = new MemoryImageSource(width, height, imageData, 0, width);
		displaySlice(1);
		mi.setAnimated(true);

		setCursor( new Cursor(Cursor.CROSSHAIR_CURSOR) );	
	}

	public void displaySlice(int sliceNumber) {

		mi.newPixels(imageData, getColorModel(),
			(sliceNumber-1)*width*height, width);

		m_image = createImage(mi);
	}

	/**
	 *  Clears the marker location.
	 *
	 */
	 public void clearMarker() {
			marker_x = -1;
			marker_y = -1;
	}

	/**
	 *  Displays the marker on the MRI image.
	 *
	 *  @param      x_in X-location for the marker
	 *  @param      y_in Y-location for the marker
	 */
	public void displayMarker( int x_in, int y_in ) {
			if( x_in >= 0 && x_in < 512 &&
				y_in >= 0 && y_in < 512 ) {
					marker_x = x_in;
					marker_y = y_in;
			}

			repaint();
	}

	public void paintComponent(Graphics g) {
	
		super.paintComponent(g);

		g.setFont( font );

//		Dimension d = super.getSize();
//		g.drawImage(m_image, 0, 0, d.width, d.height, this);

		g.drawImage(m_image, 0, 0, 2*width, 2*height, this);

		Color c = g.getColor();
		g.setColor(new Color(255, 255, 255));

		if( marker_x != -1 && marker_y != -1 ) {
			g.drawString("+", marker_x, marker_y);
		}

		g.drawString("(" + marker_x/2 + ", " + marker_y/2 + ")", 10, 10);
	}
}
