import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class BFF {

	private int nrows;
	private int ncols;
	private int nslices;
	private int nechoes;
	/* private int[] data; */
	private Volume data;
	private Vector header;

	//-----------------------------------------------------
	//
	//  Constructor
	//
	BFF(String filename) {

		read( filename );
	}

	public void read( String filename ) {

		DataInputStream ds=null;
		FileInputStream fis=null;                             
		String line;
		
		//
		//  Read header first.
		//
		header = new Vector(0);

		try {
			fis = new FileInputStream(filename);
			BufferedReader d = new BufferedReader( new InputStreamReader(fis) );

			//
			//  Read in each line of the header.
			//
			do {
				line = d.readLine();

				if( line.charAt(0) != '\014' ) {
					System.out.println(line);
					header.add( line );
				}

			} while( line.charAt(0) != '\014' );

			parseHeader();

			//
			//  Now read in the data.
			//
			fis = new FileInputStream(filename);
			ds = new DataInputStream(fis);

			do {
				line = ds.readLine();
			} while( line.charAt(0) != '\014' );

			data = new Volume(nrows, ncols, nslices*nechoes);
			data.read( ds );

		}
		catch( IOException e ) {
			System.out.println("Could not find the file.");
			System.exit(0);
		}

	}

	//-----------------------------------------------------
	//
	//  Data Returns. 
	//
    public int nRows() {
		return nrows;
	}

    public int nCols() {
		return ncols;
	}

    public int nSlices() {
		return nslices;
	}

    public int nEchoes() {
		return nechoes;
	}

	public void show() {
		System.out.println("The volume is " + nrows + "x" + ncols + "x" + nslices + "x" + nechoes);
	}

	public Volume getVolume() {
		return data;
	}

	public int[] getSlice( int slice_num ) {
		
		//
		//  Check the number of slices.
		//
		if( slice_num > nslices ) {
			System.out.println("WRong number of slices");
			System.exit(-1);
		}

		return data.getSlice( slice_num );
	}

	// Function is used to get the rows, columns etc out.
	private void parseHeader() {
		nrows = parseForInt("nrows");
		ncols = parseForInt("ncols");
		nslices = parseForInt("nslices");
		nechoes = parseForInt("nechoes");
	}

	private int parseForInt( String keyword ) {
		int toreturn = -1;
		String temp;
		Integer tempInt;

		for(int ii=0; ii<header.size(); ii++) {
			temp = (String)header.elementAt(ii);

			if( temp.indexOf(keyword) != -1 ) {
				tempInt = new Integer(temp.substring( temp.indexOf(":")+1 , 
				                                      temp.length() ).trim());
				toreturn = tempInt.intValue();
			}
		}
		return toreturn;
	}
}
