import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class DisplayMIF extends JFrame implements KeyListener, MouseListener, MouseMotionListener {

	private ImagePanel imagePanel;
	private JSlider sliceSlider;
	private JTextPane textArea;
	private int width, height, nslices;
	private int[] imageData;
	private Point mouseLoc;

	//-----------------------------------------------------
	//
	//  Constructor
	//
	DisplayMIF(String title) {

		//
		//  Tells the program to exit when this application is closed.
		//
		setDefaultCloseOperation(3);

		width = 256;
		height = 256;
		nslices = 1;

		//
		//  Scale and set the imageData variable.
		//
		imageData = new int[width*height];
		for(int row=0; row<height; row++) {
			for(int col=0; col<width; col++) {
				imageData[row*width+col] = row;
			}
		}

		//
		//  Set the true imageData variable to RGB.
		//
		int jj;
		for(int ii=0; ii<width*height; ii++) {
			jj = imageData[ii] & 0xff;
			imageData[ii] = ((255<<24) | (jj<<16) | (jj<<8) | jj);
		}

		
		//
		//  Set the mouse and key listeners.
		//
		addKeyListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);

		getContentPane().setLayout(new BorderLayout(2,2));

		//
		//  Create the JTextArea
		//
		textArea = new JTextPane();
		textArea.setPreferredSize(new Dimension( 200, height ) );
		getContentPane().add(textArea, BorderLayout.WEST);
		textArea.setBorder(BorderFactory.createTitledBorder("Info"));
		textArea.setEditable( false );

		//
		//  Create the panel.
		//
		imagePanel = new ImagePanel(imageData, width, height, nslices);
		imagePanel.setSize(width, height);
		imagePanel.setPreferredSize(new Dimension(width, height));
		getContentPane().add(imagePanel, BorderLayout.EAST);
		imagePanel.setBorder(BorderFactory.createTitledBorder("Image"));

		
		//
		//  Create the slider.
		//
		sliceSlider = new JSlider(JSlider.HORIZONTAL, 1, nslices, 1);
		imagePanel.setSize(256,10);
		getContentPane().add(sliceSlider, BorderLayout.SOUTH);
		sliceSlider.setBorder(BorderFactory.createTitledBorder("Slice Number"));

		sliceSlider.setMajorTickSpacing(5);
		sliceSlider.setMinorTickSpacing(1);
		sliceSlider.setPaintTicks(true);
		sliceSlider.setPaintLabels(true);
		sliceSlider.addChangeListener(new SliceListener());

		//
		//  Set the size and display the frame.
		//
//		setSize(260,350);  // was this 
		setSize(260+200,350);
		setTitle(title);
		setResizable(false);
		show();
	}

	protected void paintComponent(Graphics g) {
		imagePanel.paintComponent(g);
	}

	//-----------------------------------------------------
	//
	//  Mouse Listeners
	//
    public void mouseReleased(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {
        mouseLoc = e.getPoint();
		String temp = "(" + mouseLoc.x + ", " +mouseLoc.y+")";
		textArea.setText(temp);
    }

    public void mouseExited(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}

	public void mouseDragged(MouseEvent e) {
        mouseLoc = e.getPoint();
		String temp = "(" + mouseLoc.x + ", " +mouseLoc.y+")";
		textArea.setText(temp);
	}

	public void mouseMoved(MouseEvent e) {
		mouseLoc = e.getPoint();
	}

	public void keyPressed(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {
        System.out.println("The " + e.getKeyChar() + " key pressed (" + mouseLoc.x + ", " +mouseLoc.y+").");

		if( e.getKeyChar() == 'q' || e.getKeyChar() == 'Q' ) {
			System.exit(0);
		}
	}

	class SliceListener implements ChangeListener {
		
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider)e.getSource();
			imagePanel.displaySlice(source.getValue());
		}
	}

}

/*class ImagePanel extends JPanel {

	private MemoryImageSource mi;
	private MemoryImageSource mi_overlay;
	private Image m_image;
	private Image m_overlay;
	int width, height, nslices;
	int imageData[];
	int overlayData[];

	ImagePanel(int[] p_imageData, int p_width, int p_height, int p_nslices) {

		//
		//  Copy the width and height to internal variables.
		//
		width = p_width;
		height = p_height;
		nslices = p_nslices;

		imageData = new int[width*height*nslices];

		//
		//  Copy the data over.
		//
		for(int ii=0; ii<width*height*nslices;ii++) {
			imageData[ii] = p_imageData[ii];
		}

		//
		//  Create the Overlay
		//
		overlayData = new int[width*height];
		for(int ii=0; ii<width*height; ii++) {
			// overlayData[ii] = (int)((0<<24) | (0<<16) | (0<<8) | 0);
			overlayData[ii] = 0;
		}
		mi_overlay = new MemoryImageSource(width, height, overlayData, 0, width);

		mi = new MemoryImageSource(width, height, imageData, 0, width);
		displaySlice(1);
		mi.setAnimated(true);
	}

	public void displaySlice(int sliceNumber) {

		mi.newPixels(imageData, getColorModel(), 
			(sliceNumber-1)*width*height, width);

		m_image = createImage(mi);

		//
		//  Create overaly.
		//
		System.out.println("Here 4");
		mi_overlay.newPixels(overlayData, getColorModel(), 
			0, width);
		System.out.println("Here 5");
		m_overlay = createImage(mi_overlay);
	}

	public void paintComponent(Graphics g) {
		g.drawImage(m_image, 0, 0, width, height, this);
		g.drawImage(m_overlay, 0, 0, width, height, this);
	}
}
*/