package apps.stats;

import cj.appkit.*;
import cj.volume.*;
import cj.volume.measure.*;
import cj.utilities.Format;

final public class StatsApp {
	public static void main(String argv[]) { new StatsKit().main(argv); }
}

class StatsKit extends FilterAppKit {

	double percentile = 99.8;
	boolean calc_min = false;
	boolean calc_max = false;
	boolean calc_prctile = false;
	boolean calc_mean = false;
	boolean calc_noise = false;
	boolean calc_bg = false;
	NoiseEstimate ne = new NoiseEstimate();
//	Statistics st = new Statistics();

	public void pre() { filterMode = READ_ONLY; }

	public String getAppName() { return "mrstat"; }
	public String getDialogName() { return "Analyse image"; }

    protected void moreOpts()
	{
		addOpt_reqArg("percentile",'p',"Calculate <arg>th percentile");
		addOpt_noArg("minimum",'m',"Calculate minimum value");
		addOpt_noArg("maximum",'M',"Calculate maximum value");
		addOpt_noArg("mean",'u',"Calculate mean value");
		addOpt_noArg("noise",'n',"Calculate noise level");
		addOpt_noArg("bglevel",'b',"Calculate background level");
		addOpt_noArg("all",'a',"Calculate all of the above");
	}

	public void initParams(MCVolume v) {}
	public void printArgs() {}

	protected boolean parseArg(int c)
	{
			switch(c) {
				case 'p':
					percentile = getFloatArg("percentile");
					calc_prctile = true;
					break;
				case 'm':
					calc_min = true;
					break;
				case 'M':
					calc_max = true;
					break;
				case 'u':
					calc_mean = true;
					break;
				case 'n':
					calc_noise = true;
					break;
				case 'b':
					calc_bg = true;
					break;
				case 'a':
					calc_prctile = calc_min = calc_max = calc_mean
                                 = calc_noise = calc_bg = true;
					break;
				default:
					return false;
			}
		return true;
	}

	public MCVolume filter(MCVolume dm) {
		//
		//  Set kappa based on the noise estimation if it 
		//  was not specified as a parameter.
		//

		Format.print(System.out, "%15s: " + filename + "\n", "Filename");

		if( calc_min )
		{
			Format.print(System.out, "%15s: " + Statistics.min(dm) + "\n", "Minimum");
		}

		if( calc_max )
		{
			Format.print(System.out, "%15s: " + Statistics.max(dm) + "\n", "Maximum");
		}

		if( calc_mean )
		{
			Format.print(System.out, "%15s: " + Statistics.mean(dm) + "\n", "Mean");
		}

		if( calc_noise )
		{
			Format.print(System.out, "%15s: " + ne.gradientMethod(dm) + "\n", "Noise");
		}

		if( calc_bg )
		{
			Format.print(System.out, "%15s: " + ne.histogramMethod(dm) + "\n", "Background");
		}

		if( calc_prctile ) 
		{
			Format.print(System.out, "%3.1f ", percentile);
			Format.print(System.out, "%10s: " + Statistics.prctile(dm,percentile) + "\n", "Percentile");
		}

		return dm;
	}
}
