package apps.susan;

import cj.volume.*;
import cj.volume.measure.*;
import cj.volume.process.*;
import cj.appkit.FilterAppKit;

final public class SusanApp {
   public static void main(String argv[]) { new SusanKit().main(argv); }
}

final class SusanKit extends FilterAppKit {

	private double kappa = 0;
    private boolean constKappa = false;
	private int radius = 2;
	private double sigma = 2.8;
	private boolean dim2 = true;
	private int niters = 1;
	private String output_suffix = "S";

	private NoiseEstimate ne = new NoiseEstimate();
	private SusanFilter sf = null;
    private String comment = "";

	public String getAppName() { return "susan"; }
	public String getDialogName() { return "Process image"; }
    public String getOutputSuffix() { return output_suffix; }
    public String getComment() { return comment; }

    protected void moreOpts() { 
        addOpt_reqArg("kappa", 'k', "Kappa strength parameter");
        addOpt_reqArg("radius", 'r', "Half-window size in pixels (def: 2)");
        addOpt_reqArg("sigma", 's', "Gaussian kernel size (def: 2.8)");
        addOpt_reqArg("niters", 'i', "Number of iterations (def: 1).");
        addOpt_noArg("2d", '2', "Do 2D filtering.");
        addOpt_noArg("3d", '3', "Do 3D filtering.");
        addOpt_reqArg("output-suffix", 800, "Output suffix (def: 'S').");
    }

    protected boolean parseArg(int c)
    {
    	switch(c) {
			case '2':
				dim2 = true;
				break;
			case '3':
				dim2 = false;
				break;
			case 'i':
				niters = getIntArg("number of iterations");
				break;
			case 'r':
				radius = getIntArg("radius");
				break;
			case 's':
				sigma = getDoubleArg("sigma");
				break;
			case 'k':
				constKappa = true;
				kappa = getDoubleArg("kappa");
				break;
			case 800:
				output_suffix = getStringArg("suffix");
				break;
			default:
				return false;
		}
		return true;

    }

	final public void initParams(MCVolume v)
	{
		if (!constKappa) { kappa = ne.gradientMethod( v ) * 0.5; }
		sf = new SusanFilter(kappa, sigma, radius);
	}

    final public void printArgs()
   	{
		System.out.println("Number of iterations: " + niters);
		System.out.println("Kappa: " + kappa);
		System.out.println("Radius: " + radius);
		System.out.println("Sigma: " + sigma);
   	}

    final public MCVolume filter(MCVolume v)
	{
		if ( dim2 ) { 
			sf.susan2d(v, niters);
			comment = "susan2d";
		}
		else 
		{
			sf.susan3d(v, niters);
			comment = "susan3d";
		}
		comment += " kappa=" + kappa + " radius=" + radius +
                   " sigma=" + sigma + " iters=" + niters;

		return v;
	}
}
