package apps.t2lincom;

import gnu.getopt.*;
import cj.fileformats.*;
import cj.volume.*;
import cj.appkit.FilterAppKit;
import cj.mri.relax.lincom.*;

final public class T2LinComApp {
   public static void main(String argv[]) {
      T2LinComKit myApp = new T2LinComKit(); myApp.main(argv);
   }
}

final class T2LinComKit extends FilterAppKit {

	private int coef_set = 3;
	private ShortFracLincom sfl = null;
    private String comment = "t2linear combination";

	public String getAppName() { return "t2lincom"; }
	public String getDialogName() { return "Process image"; }
    public String getOutputSuffix() { return "_t2lincom"; }
    public String getComment() { return comment; }

    public void help() {
		System.out.println("t2lincom [options] <filename> [<filename> ...]");
		printOptions();
    }

    protected void moreOpts() { 
        addOpt_reqArg("set", 's', "Coefficient set (def: 3).");
    }

    protected void parseArg(int c, Getopt g)
    {
    	switch(c) {
			case 's':
				coef_set = getIntArg("set");
				break;
			default:
				help();
                System.exit(-1);
		}

    }

	final public void initParams(MRIFile b)
	{
		super.initParams(b);
		sfl = new ShortFracLincom(b.getTE());
		sfl.setVerbose( verbose );
	}
	
	final public void preSave(MRIFile b)
	{
		super.preSave(b);

		b.setDataType(MRIFile.DATATYPE_FLOAT);
	}

    final public void printArgs()
   	{
		System.out.println("Coefficient set is " + coef_set);
   	}

    final public MCVolume filter(MCVolume v)
	{
		Volume vout = sfl.calculateShortFrac(v);

		MCVolume newmcvol = new MCVolume(v.getNRows(), v.getNCols(), v.getNSlices(), 1);
		newmcvol.setVolume(vout, 0);

		return newmcvol;
	}
}
